import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _typeof from "@babel/runtime/helpers/typeof";
import React, { useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import useFieldApi from "@data-driven-forms/react-form-renderer/use-field-api";
import DataDrivenSelect from '@data-driven-forms/common/select';
import fnToString from '@data-driven-forms/common/utils/fn-to-string';
import CarbonSelect from "carbon-components-react/es/components/Select/Select.js";
import MultiSelect from "carbon-components-react/es/components/MultiSelect/MultiSelect.js";
import SelectItem from "carbon-components-react/es/components/SelectItem/SelectItem.js";
import ComboBox from "carbon-components-react/es/components/ComboBox/ComboBox.js";
import prepareProps from '../prepare-props';
export var multiOnChange = function multiOnChange(input, simpleValue) {
  return function (_ref) {
    var selectedItem = _ref.selectedItem,
        selectedItems = _ref.selectedItems;

    if (simpleValue) {
      return input.onChange((selectedItems === null || selectedItems === void 0 ? void 0 : selectedItems.map(function (_ref2) {
        var value = _ref2.value;
        return value;
      })) || selectedItem.value);
    } else {
      return input.onChange(selectedItems || selectedItem);
    }
  };
};
export var getMultiValue = function getMultiValue(value, options) {
  return (Array.isArray(value) ? value : value ? [value] : []).map(function (item) {
    return _typeof(item) === 'object' ? item : options.find(function (_ref3) {
      var value = _ref3.value;
      return value === item;
    });
  });
};

var ClearedMultiSelectFilterable = function ClearedMultiSelectFilterable(_ref4) {
  var invalidText = _ref4.invalidText,
      hideSelectedOptions = _ref4.hideSelectedOptions,
      noOptionsMessage = _ref4.noOptionsMessage,
      onInputChange = _ref4.onInputChange,
      options = _ref4.options,
      isFetching = _ref4.isFetching,
      invalid = _ref4.invalid,
      isMulti = _ref4.isMulti,
      classNamePrefix = _ref4.classNamePrefix,
      closeMenuOnSelect = _ref4.closeMenuOnSelect,
      onChange = _ref4.onChange,
      originalOnChange = _ref4.originalOnChange,
      carbonLabel = _ref4.carbonLabel,
      placeholder = _ref4.placeholder,
      isDisabled = _ref4.isDisabled,
      rest = _objectWithoutProperties(_ref4, ["invalidText", "hideSelectedOptions", "noOptionsMessage", "onInputChange", "options", "isFetching", "invalid", "isMulti", "classNamePrefix", "closeMenuOnSelect", "onChange", "originalOnChange", "carbonLabel", "placeholder", "isDisabled"]);

  return /*#__PURE__*/React.createElement(MultiSelect.Filterable, _extends({
    disabled: isDisabled
  }, rest, {
    placeholder: carbonLabel || placeholder,
    onChange: originalOnChange,
    titleText: rest.labelText,
    id: rest.name,
    invalid: Boolean(invalidText),
    invalidText: invalidText,
    items: options,
    initialSelectedItems: getMultiValue(rest.value, options)
  }));
};

ClearedMultiSelectFilterable.propTypes = {
  invalidText: PropTypes.node,
  hideSelectedOptions: PropTypes.any,
  noOptionsMessage: PropTypes.any,
  onInputChange: PropTypes.func,
  options: PropTypes.array,
  isFetching: PropTypes.bool,
  invalid: PropTypes.oneOfType([PropTypes.bool, PropTypes.node]),
  isMulti: PropTypes.bool,
  classNamePrefix: PropTypes.any,
  closeMenuOnSelect: PropTypes.any,
  onChange: PropTypes.func,
  originalOnChange: PropTypes.func,
  carbonLabel: PropTypes.node,
  placeholder: PropTypes.node,
  isRequired: PropTypes.bool,
  isDisabled: PropTypes.bool
};

var ClearedMultiSelect = function ClearedMultiSelect(_ref5) {
  var invalidText = _ref5.invalidText,
      hideSelectedOptions = _ref5.hideSelectedOptions,
      noOptionsMessage = _ref5.noOptionsMessage,
      onInputChange = _ref5.onInputChange,
      options = _ref5.options,
      isFetching = _ref5.isFetching,
      invalid = _ref5.invalid,
      isMulti = _ref5.isMulti,
      classNamePrefix = _ref5.classNamePrefix,
      closeMenuOnSelect = _ref5.closeMenuOnSelect,
      onChange = _ref5.onChange,
      originalOnChange = _ref5.originalOnChange,
      carbonLabel = _ref5.carbonLabel,
      placeholder = _ref5.placeholder,
      isDisabled = _ref5.isDisabled,
      rest = _objectWithoutProperties(_ref5, ["invalidText", "hideSelectedOptions", "noOptionsMessage", "onInputChange", "options", "isFetching", "invalid", "isMulti", "classNamePrefix", "closeMenuOnSelect", "onChange", "originalOnChange", "carbonLabel", "placeholder", "isDisabled"]);

  return /*#__PURE__*/React.createElement(MultiSelect, _extends({
    disabled: isDisabled
  }, rest, {
    label: carbonLabel || placeholder,
    onChange: originalOnChange,
    titleText: rest.labelText,
    id: rest.name,
    invalid: Boolean(invalidText),
    invalidText: invalidText,
    items: options,
    initialSelectedItems: getMultiValue(rest.value, options)
  }));
};

ClearedMultiSelect.propTypes = {
  invalidText: PropTypes.node,
  hideSelectedOptions: PropTypes.any,
  noOptionsMessage: PropTypes.any,
  onInputChange: PropTypes.func,
  options: PropTypes.array,
  isFetching: PropTypes.bool,
  invalid: PropTypes.oneOfType([PropTypes.bool, PropTypes.node]),
  isMulti: PropTypes.bool,
  classNamePrefix: PropTypes.any,
  closeMenuOnSelect: PropTypes.any,
  onChange: PropTypes.func,
  originalOnChange: PropTypes.func,
  carbonLabel: PropTypes.node,
  placeholder: PropTypes.node,
  isRequired: PropTypes.bool,
  isDisabled: PropTypes.bool
};

var getSelectValue = function getSelectValue(value, isMulti) {
  return isMulti ? value : Array.isArray(value) ? value[0] : value;
};

var ClearedSelect = function ClearedSelect(_ref6) {
  var isSearchable = _ref6.isSearchable,
      isClearable = _ref6.isClearable,
      isDisabled = _ref6.isDisabled,
      isMulti = _ref6.isMulti,
      invalidText = _ref6.invalidText,
      hideSelectedOptions = _ref6.hideSelectedOptions,
      noOptionsMessage = _ref6.noOptionsMessage,
      onInputChange = _ref6.onInputChange,
      options = _ref6.options,
      isFetching = _ref6.isFetching,
      invalid = _ref6.invalid,
      classNamePrefix = _ref6.classNamePrefix,
      closeMenuOnSelect = _ref6.closeMenuOnSelect,
      originalOnChange = _ref6.originalOnChange,
      placeholder = _ref6.placeholder,
      value = _ref6.value,
      rest = _objectWithoutProperties(_ref6, ["isSearchable", "isClearable", "isDisabled", "isMulti", "invalidText", "hideSelectedOptions", "noOptionsMessage", "onInputChange", "options", "isFetching", "invalid", "classNamePrefix", "closeMenuOnSelect", "originalOnChange", "placeholder", "value"]);

  return /*#__PURE__*/React.createElement(CarbonSelect, _extends({
    value: getSelectValue(value, isMulti),
    disabled: isFetching
  }, rest, {
    id: rest.name,
    invalid: Boolean(invalidText),
    invalidText: invalidText
  }), isFetching && /*#__PURE__*/React.createElement(SelectItem, {
    text: placeholder,
    value: ''
  }), options.map(function (option, index) {
    return /*#__PURE__*/React.createElement(SelectItem, _extends({
      key: option.value || index,
      text: option.label
    }, option));
  }));
};

ClearedSelect.propTypes = {
  invalidText: PropTypes.node,
  hideSelectedOptions: PropTypes.any,
  noOptionsMessage: PropTypes.any,
  onInputChange: PropTypes.func,
  options: PropTypes.array,
  isFetching: PropTypes.bool,
  invalid: PropTypes.oneOfType([PropTypes.bool, PropTypes.node]),
  isMulti: PropTypes.bool,
  classNamePrefix: PropTypes.any,
  closeMenuOnSelect: PropTypes.any,
  onChange: PropTypes.func,
  originalOnChange: PropTypes.func,
  carbonLabel: PropTypes.node,
  placeholder: PropTypes.node,
  isDisabled: PropTypes.bool,
  isRequired: PropTypes.bool,
  isSearchable: PropTypes.bool,
  isClearable: PropTypes.bool,
  value: PropTypes.any
};

var getComboInitialValue = function getComboInitialValue(value) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var result = Array.isArray(value) ? value[0] : value;
  return _typeof(result) === 'object' ? result : options.find(function (_ref7) {
    var value = _ref7.value;
    return value === result;
  }) || result;
};
/**
 * Combobox cannot be multi value
 */


var ClearedSelectSearchable = function ClearedSelectSearchable(_ref8) {
  var isSearchable = _ref8.isSearchable,
      isClearable = _ref8.isClearable,
      isDisabled = _ref8.isDisabled,
      isMulti = _ref8.isMulti,
      invalidText = _ref8.invalidText,
      hideSelectedOptions = _ref8.hideSelectedOptions,
      noOptionsMessage = _ref8.noOptionsMessage,
      onInputChange = _ref8.onInputChange,
      _ref8$options = _ref8.options,
      options = _ref8$options === void 0 ? [] : _ref8$options,
      isFetching = _ref8.isFetching,
      invalid = _ref8.invalid,
      classNamePrefix = _ref8.classNamePrefix,
      closeMenuOnSelect = _ref8.closeMenuOnSelect,
      originalOnChange = _ref8.originalOnChange,
      placeholder = _ref8.placeholder,
      labelText = _ref8.labelText,
      rest = _objectWithoutProperties(_ref8, ["isSearchable", "isClearable", "isDisabled", "isMulti", "invalidText", "hideSelectedOptions", "noOptionsMessage", "onInputChange", "options", "isFetching", "invalid", "classNamePrefix", "closeMenuOnSelect", "originalOnChange", "placeholder", "labelText"]);

  return /*#__PURE__*/React.createElement(ComboBox, _extends({
    disabled: isFetching
  }, rest, {
    id: rest.name,
    invalid: Boolean(invalidText),
    invalidText: invalidText,
    initialSelectedItem: getComboInitialValue(rest.value, options),
    items: options,
    placeholder: placeholder,
    titleText: labelText,
    onChange: originalOnChange
  }));
};

ClearedSelectSearchable.propTypes = {
  invalidText: PropTypes.node,
  hideSelectedOptions: PropTypes.any,
  noOptionsMessage: PropTypes.any,
  onInputChange: PropTypes.func,
  options: PropTypes.array,
  isFetching: PropTypes.bool,
  invalid: PropTypes.oneOfType([PropTypes.bool, PropTypes.node]),
  isMulti: PropTypes.bool,
  classNamePrefix: PropTypes.any,
  closeMenuOnSelect: PropTypes.any,
  onChange: PropTypes.func,
  originalOnChange: PropTypes.func,
  carbonLabel: PropTypes.node,
  placeholder: PropTypes.node,
  isDisabled: PropTypes.bool,
  isRequired: PropTypes.bool,
  isSearchable: PropTypes.bool,
  isClearable: PropTypes.bool,
  labelText: PropTypes.string
};

var Select = function Select(props) {
  var _useFieldApi = useFieldApi(prepareProps(props)),
      isMulti = _useFieldApi.isMulti,
      isSearchable = _useFieldApi.isSearchable,
      isClearable = _useFieldApi.isClearable,
      loadOptions = _useFieldApi.loadOptions,
      input = _useFieldApi.input,
      meta = _useFieldApi.meta,
      validateOnMount = _useFieldApi.validateOnMount,
      helperText = _useFieldApi.helperText,
      rest = _objectWithoutProperties(_useFieldApi, ["isMulti", "isSearchable", "isClearable", "loadOptions", "input", "meta", "validateOnMount", "helperText"]);

  var _useState = useState(0),
      _useState2 = _slicedToArray(_useState, 2),
      loadOptionsChangeCounter = _useState2[0],
      setCounter = _useState2[1];

  var loadOptionsStr = fnToString(loadOptions);
  useEffect(function () {
    setCounter(loadOptionsChangeCounter + 1); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [loadOptionsStr]);
  var isSearchClear = isSearchable || isClearable;
  var Component = isMulti && isSearchClear ? ClearedMultiSelectFilterable : isMulti ? ClearedMultiSelect : isSearchClear ? ClearedSelectSearchable : ClearedSelect;
  var invalidText = (meta.touched || validateOnMount) && (meta.error || meta.submitError) || '';
  var text = (meta.touched || validateOnMount) && meta.warning || helperText;
  return /*#__PURE__*/React.createElement(DataDrivenSelect, _extends({
    SelectComponent: Component,
    simpleValue: false
  }, rest, input, {
    isMulti: isMulti,
    loadOptions: loadOptions,
    invalidText: invalidText,
    loadOptionsChangeCounter: loadOptionsChangeCounter,
    originalOnChange: multiOnChange(input, rest.simpleValue),
    helperText: text
  }));
};

Select.propTypes = {
  isDisabled: PropTypes.bool,
  isRequired: PropTypes.bool,
  options: PropTypes.arrayOf(PropTypes.shape({
    value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    label: PropTypes.node
  }))
};
Select.defaultProps = {
  loadingMessage: 'Loading...'
};
export default Select;